@@ -5,12 +5,11 @@
 $::PROTOCOL = '(?:s?https?|ftp)';
 $::RXURL    = '(?:s?https?|ftp)://[-\\w.!~*\'();/?:@&=+$,%#]+' ;
 $::charset  = 'EUC-JP';
-$::version  = '1.1.0_1 ($Date: 2006/07/03 16:41:49 $)';
+$::version  = '1.1.2_0 ($Date: 2006/07/29 12:00:56 $)';
 %::form     = ();
-$::me       = $::postme = 'http://'.$ENV{HTTP_HOST}.$ENV{SCRIPT_NAME};
+$::me       = $::postme = $ENV{SCRIPT_NAME};
 $::print    = ' 'x 10000; $::print = '';
 %::config   = ( crypt => '' , sitename => 'wifky!' );
-$::psuffix  = '.pl';
 
 binmode(STDOUT);
 binmode(STDIN);
@@ -26,7 +25,7 @@ eval{
     }
     &load_config;
     &init_globals;
-    foreach my $pl (sort grep(/$::psuffix$/o,&directory) ){
+    foreach my $pl (sort grep(/\.pl$/,&directory) ){
         do $pl; $@ and die($@);
     }
 
@@ -38,8 +37,8 @@ eval{
         }else{ # output page itself.
             &action_view($::form{p});
         }
-    }elsif( &object_exists('FrontPage') ){ # default(no-option)
-        &action_view('FrontPage');
+    }elsif( &object_exists($::config{FrontPage}) ){
+        &action_view($::config{FrontPage});
     }else{
         &do_index('recent','rindex','-l');
     }
@@ -69,6 +68,8 @@ sub init_globals{
     $::target = ( $::config{target}
                 ? sprintf(' target="%s"',$::config{target}) : '' );
     $::config{CSS} ||= 'CSS';
+    $::config{FrontPage} ||= 'FrontPage';
+
     %::inline_plugin = (
         'adminmenu'=> \&plugin_menubar ,
         'menubar'  => \&plugin_menubar ,
@@ -132,8 +133,8 @@ sub init_globals{
     @::body_header = <DATA>;
 
     @::menubar = (
-        &anchor( 'FrontPage' , undef  ) ,
-        &anchor( 'New'       , { a=>'new' } ) ,
+        &anchor( $::config{FrontPage} , undef  ) ,
+        &anchor( 'New'                , { a=>'new' } ) ,
     );
 
     ### menubar ###
@@ -142,14 +143,14 @@ sub init_globals{
             push(@::menubar,
                 &anchor('Edit',{
                     a=>'edt',
-                    p=>( exists $::form{p} ? $::form{p} : 'FrontPage' )
+                    p=>( exists $::form{p} ? $::form{p} : $::config{FrontPage} )
                 },{
                     rel=>'nofollow'
                 }) );
         push( @::menubar ,
             &anchor('Edit(Admin)',{
                 a=>'edt',
-                p=>( exists $::form{p} ? $::form{p} : 'FrontPage' ),
+                p=>( exists $::form{p} ? $::form{p} : $::config{FrontPage} ),
                 admin=>'admin'
             },{rel=>'nofollow'}) );
     }
@@ -168,11 +169,12 @@ sub init_globals{
         ' General Options' => [
             { desc=>'the sitename', name=>'sitename', size=>40 },
             { desc=>'enable link to file://...', name=>'locallink' ,
-              type=>'checkbox', size=>3 },
+              type=>'checkbox' },
             { desc=>'forbid any one but administrator creating a new page.' ,
               name=>'lonely' , type=>'checkbox' },
             { desc=>'target value for external link.',name=>'target'},
             { desc=>'pagename or url for CSS' , name=>'CSS' , size=>40 },
+            { desc=>'pagename for FrontPage'  , name=>'FrontPage' , size=>40 },
         ],
         ' Section Marks' => [
             { desc=>'section mark', name=>'sectionmark', size=>3 } ,
@@ -181,20 +183,29 @@ sub init_globals{
         ]
     );
 
-    @::inline_syntax_plugin = (
-        \&preprocess_innerlink1  , \&preprocess_innerlink2 ,
-        \&preprocess_outerlink1  , \&preprocess_outerlink2 ,
-        \&preprocess_attachment  , \&preprecess_htmltag ,
-        \&preprocess_decorations , \&preprocess_plugin ,
-        \&preprocess_rawurl
+    %::inline_syntax_plugin = (
+        '100_innerlink1' => \&preprocess_innerlink1 ,
+        '200_innerlink2' => \&preprocess_innerlink2 ,
+        '300_outerlink1' => \&preprocess_outerlink1  ,
+        '400_outerlink2' => \&preprocess_outerlink2 ,
+        '500_attachment' => \&preprocess_attachment  ,
+        '600_htmltag'    => \&preprecess_htmltag ,
+        '700_decoration' => \&preprocess_decorations ,
+        '800_plugin'     => \&preprocess_plugin ,
+        '900_rawurl'     => \&preprocess_rawurl ,
     );
 
-    @::block_syntax_plugin = (
-        \&block_listing   , \&block_definition ,
-        \&block_midashi1  , \&block_midashi2 ,
-        \&block_centering , \&block_quoting ,
-        \&block_table     , \&block_quote_center ,
-        \&block_separator , \&block_normal
+    %::block_syntax_plugin = (
+        '100_list'       => \&block_listing   ,
+        '200_definition' => \&block_definition ,
+        '300_midashi1'   => \&block_midashi1  ,
+        '400_midashi2'   => \&block_midashi2 ,
+        '500_centering'  => \&block_centering ,
+        '600_quoting'    => \&block_quoting ,
+        '700_table'      => \&block_table ,
+        '800_htmltag'    => \&block_htmltag ,
+        '900_seperator'  => \&block_separator ,
+        '990_normal'     => \&block_normal ,
     );
     @::footer_plugin = ( \&plugin_footnote_flush );
 }
@@ -306,19 +317,21 @@ sub deyen{
 }
 
 sub mtime{
-    unless( exists $::mtime_cache{ $_[0] } ){
-        my @stat=(stat($_[0])) or return '';
+    my $fn=shift;
+    unless( exists $::mtime_cache{$fn} ){
+        my @stat=stat($fn) 
+            or return ($::mtime_cache{$fn}='0000/00/00 00:00:00');
         my @tm=localtime($stat[9]);
-        $::mtime_cache{ $_[0] } = sprintf('%04d/%02d/%02d %02d:%02d:%02d'
+        $::mtime_cache{ $fn } = sprintf('%04d/%02d/%02d %02d:%02d:%02d'
             , 1900+$tm[5],1+$tm[4],@tm[3,2,1,0]);
     }
-    $::mtime_cache{ $_[0] };
+    $::mtime_cache{$fn};
 }
 
 sub cacheoff{
     undef %::mtime_cache;
-    undef @::work_directory_cache;
-    undef %::work_directory_cache;
+    undef @::dir_cache;
+    undef %::dir_cache;
 }
 sub title2mtime{
     &mtime( &title2fname(@_) );
@@ -364,7 +377,8 @@ sub print_form{
     my ($title,$html,$stamp) = @_;
 
     &putenc('<div class="update"
-        ><form action="%s" enctype="multipart/form-data" method="post"
+        ><form name="editform" action="%s" 
+         enctype="multipart/form-data" method="post"
         ><input type="hidden" name="stamp" value="%s"
         ><input type="hidden" name="p" value="%s"><br>'
         , $::postme,$stamp,$title );
@@ -374,36 +388,35 @@ sub print_form{
     my $fname=&title2fname( $::form{p} );
     &puts('<input type="submit" name="a" value="Preview">');
     if( exists $::form{admin} || &is_frozen() ){
-        &puts('Administrator\'s Sign:
-        <input type="password" name="password"
+        &puts('Sign: <input type="password" name="password"
         ><input type="checkbox" name="to_freeze" value="1"');
         &is_frozen() and &puts('checked');
         &puts(' >freeze <input type="hidden" name="admin" value="admin">');
     }
     &puts('<input type="submit" name="a" value="Commit">');
-    if( &object_exists($::form{p}) ){
-        &puts('<h2>Attachment</h2
-        ><p>New:<input type="file" name="butsu" size="48">');
-        &is_frozen() and &puts('Sign:<input type="password" name="qassword">');
-        &puts('<input type="submit" name="a" value="Upload"></p>');
 
-        my @attachments=&list_attachment( $::form{p} );
-        if( scalar(@attachments) > 0 ){
-            &puts('<p>');
-            foreach my $attach (sort @attachments){
-                &putenc('<input type="radio" name="f" value="%s" ><input
-                        type="text" name="dummy" readonly
-                        value="&lt;&lt;{%s}" size="20"
-                        style="font-family:monospace">'
-                          ,$attach ,$attach );
+    &puts('<h2>Attachment</h2>
+    <p>New:<input type="file" name="butsu" size="48">');
+    if( exists $::form{admin} || &is_frozen() ){
+        &puts('Sign:<input type="password" name="qassword">');
+    }
+    &puts('<input type="submit" name="a" value="Upload"></p>');
+    
+    if( my @attachments=&list_attachment( $::form{p} ) ){
+        &puts('<p>');
+        foreach my $attach (sort @attachments){
+            &putenc('<input type="radio" name="f" value="%s" ><input
+                    type="text" name="dummy" readonly value="&lt;&lt;{%s}"
+                    size="20" style="font-family:monospace"
+                    onClick="this.select();">'
+                  ,$attach ,$attach );
 
-                my $fn = &title2fname($::form{p}, $attach);
-                &putenc('(%d bytes, at %s)',(stat $fn)[7],&mtime($fn));
-                &puts('<br>');
-            }
-            &puts('</p><input type="submit" name="a" value="Delete">
-                <input type="submit" name="dummybotton" value="Download">');
+            my $fn = &title2fname($::form{p}, $attach);
+            &putenc('(%d bytes, at %s)',(stat $fn)[7],&mtime($fn));
+            &puts('<br>');
         }
+        &puts('</p><input type="submit" name="a" value="Delete">
+            <input type="submit" name="dummybotton" value="Download">');
     }
     &puts(@::formtext);
     &puts('</form></div>');
@@ -474,7 +487,7 @@ sub print_copyright{
 }
 
 sub is_frozen{
-    if( -r &title2fname(exists $::form{p} ? $::form{p} : 'FrontPage' ) ){
+    if( -r &title2fname(exists $::form{p} ? $::form{p} : $::config{FrontPage})){
         ! -w _;
     }else{
         &is('lonely');
@@ -482,10 +495,9 @@ sub is_frozen{
 }
 
 sub ninsho{
-    if( $::config{crypt} ne '' &&
-        crypt($::form{password},'wk') ne $::config{crypt} &&
-        crypt($::form{qassword},'wk') ne $::config{crypt} )
-    {
+    my $pwd = $::form{password};
+    exists $::form{qassword} and $pwd .= $::form{qassword};
+    if( $::config{crypt} ne '' && crypt($pwd,'wk') ne $::config{crypt} ){
         die('!Administrator\'s Sign is wrong!');
     }
 }
@@ -593,8 +605,11 @@ sub action_query_delete{
 
 sub action_commit{
     my $lock=&title2fname($::form{p},'LOCK');
+    if( ! mkdir($lock,0777) ){
+        &do_preview( &errmsg("!File writing conflict (lockfile=$lock)!") );
+        return;
+    }
     eval{
-        mkdir $lock,0777 or die('!File writing conflict!');
         &check_frozen;
         &check_conflict;
     };
@@ -699,22 +714,20 @@ sub action_preferences{
 
 sub action_rename{
     &ninsho;
+
     my $newtitle = $::form{newtitle};
     my $title    = $::form{p};
     my $fname    = &title2fname($title);
     my $newfname = &title2fname($newtitle);
 
-    if( -f $newfname ){
-        die("!The new page name '$newtitle' is already used.!");
-    }
-
-    foreach my $fn ( &directory() ){
-        my @p=split('__',$fn);
-        if( $p[0] eq $fname ){
-            $p[0] = $newfname;
-            rename( $fn , join('__',@p) );
-        }
+    my @list;
+    foreach my $suffix ( @{$::dir_cache{$fname}} ){
+        my $older=$fname    . $suffix ;
+        my $newer=$newfname . $suffix ;
+        -f $newfname and die("!The new page name '$newtitle' is already used.!");
+        push(@list, [ $older , $newer ] );
     }
+    rename( $_->[0] , $_->[1] ) foreach @list;
     &transfer_page($newtitle);
 }
 
@@ -729,8 +742,7 @@ sub action_seek{
         my $title  = &fname2title( $fn );
         if( index($title ,$keyword) >= 0 ){
             &puts('<li>' . &anchor($title,{ p=>$title }) . ' (title)</li>');
-        }else{
-            open(FP,$fn) or die("Can not open the file $fn");
+        }elsif( open(FP,$fn) ){
             while( <FP> ){
                 if( index($_,$keyword) >= 0 ){
                     &puts('<li>' . &anchor($title,{ p=>$title } ) . '</li>' );
@@ -749,7 +761,8 @@ sub action_seek{
 sub action_delete{
     if( exists $::form{yes} ){
         &is_frozen() and &ninsho;
-        unlink( &title2fname( $::form{p} , $::form{f} ) );
+        my $fn=&title2fname( $::form{p} , $::form{f} );
+        unlink( $fn ) or rmdir( $fn );
         &cacheoff;
     }
     &do_preview;
@@ -823,8 +836,7 @@ sub do_submit{
 
 sub transfer_url{
     my $url=(shift || $::me);
-    &puts('Content-type: text/html');
-    &puts('');
+    &puts('Content-type: text/html','');
     &puts(qq(<html><head><title>Moving...</title
         ><meta http-equiv="refresh" content="1;URL=${url}"
         ></head><body
@@ -854,23 +866,21 @@ sub do_preview{
 sub action_edit{
     my $title = $::form{p};
     &print_header(divclass=>'max',title=>'Edit');
-    &putenc('<h1>Edit: %s</h1>',$title);
+    &begin_day("Edit: $title");
     &print_form( $title , &enc(&read_object( $title )), &title2mtime($title) );
 
     if( &object_exists($::form{p}) && exists $::form{admin} ){
         &putenc('<h2>Rename</h2>
-            <p>
-            <form action="%s" method="post">
+            <p><form action="%s" method="post">
             <input type="hidden"  name="a" value="ren">
             <input type="hidden"  name="p" value="%s">
             Title: <input type="text" name="newtitle" value="%s" size="80">
-            <br>Administrator\'s Sign:
-            <input type="password" name="password">
+            <br>Sign: <input type="password" name="password">
             <br><input type="submit" name="ren" value="Submit">
-            </form>
-            </p>
+            </form></p>
         ' , $::postme , $::form{p} , $::form{p} );
     }
+    &end_day();
     &print_footer;
 }
 
@@ -910,30 +920,35 @@ sub action_cat{
     close(FP);
 }
 
-sub directory{
-    unless( defined(@::work_directory_cache) ){
+sub cache_update{
+    unless( defined(@::dir_cache) ){
         opendir(DIR,'.') or die('can\'t read work directory.');
-        grep( ($::work_directory_cache{ $_ } = 1,0)
-            ,  @::work_directory_cache = readdir(DIR) );
+        while( my $fn=readdir(DIR) ){
+            push( @::dir_cache , $fn );
+            $fn =~ /^((?:[0-9a-f][0-9a-f])+)(__(?:[0-9a-f][0-9a-f])+)?$/
+                and push( @{$::dir_cache{$1}} , $2 );
+        }
         closedir(DIR);
     }
-    @::work_directory_cache;
+}
+
+sub directory{
+    &cache_update() ; @::dir_cache;
 }
 
 sub list_page{
-    grep( /^([0-9a-z][0-9a-z])+$/ , &directory );
+    &cache_update() ; keys %::dir_cache;
 }
 
 sub object_exists{
-      defined( %::work_directory_cache )
-    ? exists $::work_directory_cache{ &title2fname($_[0]) }
-    : -f &title2fname($_[0]) ;
+    &cache_update() ; exists $::dir_cache{ &title2fname($_[0]) }
 }
 
 sub list_attachment{
-    my $prefix = &title2fname( shift ) . '__';
-    my $off = length($prefix);
-    map(&fname2title(substr($_,$off)),grep(index($_,$prefix)==0,&directory));
+    &cache_update();
+    map(  &fname2title(substr($_,2))
+        , grep(defined($_) && /^__/,
+               @{ $::dir_cache{&title2fname( shift ) } } ) );
 }
 
 sub print_page{
@@ -1021,7 +1036,11 @@ sub plugin_footnote{
     $footnotetext =~ s/\<[^\>]*\>//g;
 
     my $i=$#{$session->{footnotes}} + 1;
-    qq(<sup><a href="#ft${i}" name="fm${i}" title="${footnotetext}">*${i}</a></sup>);
+    my %attr=( title=>$footnotetext );
+    $session->{index} and $attr{name}="fm${i}";
+    '<sup>' . 
+    &anchor("*${i}", { p=> $::form{p} } , \%attr , "#ft${i}" ) .
+    '</sup>' ;
 }
 
 sub plugin_footnote_flush{
@@ -1033,8 +1052,11 @@ sub plugin_footnote_flush{
     &puts(qq(<div class="footnote">));
     foreach my $t (@{$footnotes}){
         ++$i;
-        &puts(qq(<p class="footnote"><a name="ft${i}" href="#fm${i}"
-                >*${i}</a> $t</p>));
+        &puts('<p class="footnote">' ,
+            &anchor("*$i",{ p=>$::form{p} } , 
+            ($session->{index} ? { name=>"ft$i"} : undef) ,
+            "#fm$i"),
+            "$t</p>");
     }
     &puts(qq(</div><!--footnote-->));
     delete $session->{footnotes};
@@ -1125,13 +1147,22 @@ sub plugin_comment{
         close(FP);
     }
     unless( exists $opt{f} ){
-        $buf .= qq(<form action="$::postme" method="post"
-        ><input type="hidden" name="p" value="$etitle"
-        ><input type="hidden" name="a" value="comment"
-        ><input type="hidden" name="comid" value="$ecomid"
-        ><input type="text" name="who" size="10"
-        ><input type="text" name="comment" size="60"
-        ><input type="submit" name="Comment" value="Comment"></form>);
+        $buf .= <<HTML
+<form action="$::postme" method="post" class="comment">
+<input type="hidden" name="p" value="$etitle">
+<input type="hidden" name="a" value="comment">
+<input type="hidden" name="comid" value="$ecomid">
+<div class="field name">
+<input type="text" name="who" size="10" class="field">
+</div>
+<div class="textarea">
+<textarea name="comment" cols="60" rows="1" class="field"></textarea>
+</div>
+<div class="button">
+<input type="submit" name="Comment" value="Comment">
+</div>
+</form>
+HTML
     }
     $buf . '</div></div>';
 }
@@ -1144,7 +1175,7 @@ sub plugin_pagename{
     }elsif( exists $::form{keyword} ){
         &enc('Seek: '.$::form{keyword});
     }else{
-        &enc( exists $::form{p} ? $::form{p} : 'FrontPage' );
+        &enc( exists $::form{p} ? $::form{p} : $::config{FrontPage} );
     }
 }
 
@@ -1223,8 +1254,8 @@ sub preprocess_rawurl{
 
 sub preprocess{
     my ($text,$session) = @_;
-    foreach my $p ( @::inline_syntax_plugin ){
-        $p->( \$text , $session );
+    foreach my $p ( sort keys %::inline_syntax_plugin ){
+        $::inline_syntax_plugin{$p}->( \$text , $session );
     }
     $text;
 }
@@ -1250,7 +1281,7 @@ sub midashi{
                   , { p     => $session->{title} }
                   , { class => "${cls}mark sanchor" }
                   , "#p${tag}"
-                  ) . $text ;
+                  ) . qq(<span class="${cls}title">$text</span>) ;
 
         if( $session->{main} ){
             &puts(qq(<div class="${cls} x${cls}">));
@@ -1281,8 +1312,8 @@ sub default_syntax_engine{
     &verbatim( $ref2html );
 
     foreach my $fragment( split(/\r?\n\r?\n/,$$ref2html) ){
-        foreach my $proc (@main::block_syntax_plugin){
-            $proc->($fragment,$session) and last;
+        foreach my $p (sort keys %::block_syntax_plugin){
+            $::block_syntax_plugin{$p}->($fragment,$session) and last;
         }
     }
     exists $session->{section} and
@@ -1377,7 +1408,7 @@ sub block_table{ ### || ... | ... |
     1;
 }
 
-sub block_quote_center{ ### <blockquote> or <center>
+sub block_htmltag{ ### <blockquote> or <center>
     my ($fragment,$session)=@_;
     return 0 unless
         $fragment =~ /\A\s*&lt;(blockquote|center)&gt;(.*)&lt;\/\1&gt;\s*\Z/si ;
